/*
 * Borders.java
 *
 * Created on 22. Dezember 2007, 08:42
 */

package name.dhiller.jBorderLiner;

import com.sun.star.container.XEnumerationAccess;
import com.sun.star.container.XEnumeration;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.AnyConverter;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.frame.XController;
import com.sun.star.table.BorderLine;
import com.sun.star.table.BorderLine2;
import com.sun.star.table.TableBorder;
import com.sun.star.table.XCell;
import com.sun.star.table.XCellRange;
import com.sun.star.text.XTextTable;
import com.sun.star.text.XTextViewCursorSupplier;
import com.sun.star.text.XTextViewCursor;
import com.sun.star.text.XTextTableCursor;

import java.awt.HeadlessException;
import java.lang.Math; 

/** Borders handling
 * 
 * @author Dietmar Hiller (Dhiller)
 * @version 120204
 */
public class Borders {
                            // values of sApplyTo from jBorderLiner.TB_ApplyTo
    private final static short APPLYTO_CELL = 0;
    private final static short APPLYTO_TABLE = 1;
    
    /** Creates a new instance of Borders */
    public Borders() {
    }

    /** DoBorders - get or set Borders
     * @param xFrame
     * @param Modifiers com.sun.star.awt.KeyModifier.SHIFT, .MOD1 (Ctrl), .MOD2 (Alt)
     * @param KeyCode com.sun.star.awt.Key.NUM1 ... .NUM9
     */
    public static void DoBorders (com.sun.star.frame.XFrame xFrame, 
            short Modifiers, short KeyCode) {
        short sApplyTo = jBorderLiner.TB_ApplyTo.GetApplyTo(); // 0: Cell, 1: Table
        Object oRange = GetTheCell (xFrame, sApplyTo);
        if (oRange != null) {
            switch (Modifiers) {
                case com.sun.star.awt.KeyModifier.MOD1: // Ctrl
                    switch (sApplyTo) {
                        case APPLYTO_CELL:
                            GetBordersCell(oRange, Modifiers, KeyCode);
                            break;
                        case APPLYTO_TABLE:
                            GetBordersTable(oRange, Modifiers, KeyCode);
                            break;     
                    }
                    break;
                case com.sun.star.awt.KeyModifier.MOD2: // Alt  
                case com.sun.star.awt.KeyModifier.SHIFT:
                    switch (sApplyTo) {
                        case APPLYTO_CELL:
                            SetBordersCell(oRange, Modifiers, KeyCode);
                            break;
                        case APPLYTO_TABLE:
                            SetBordersTable(oRange, Modifiers, KeyCode);
                            break;     
                    }
                break;
            }
        }
    }

    /**
     * Cells the border shall be applied to 
     * @param: sApplyTo {APPLYTO_CELL = 0, APPLYTO_TABLE = 1}
     * @exception ex.printStackTrace() IllegalArgumentException, 
     *      UnknownPropertyException, WrappedTargetException
     */
    private static Object GetTheCell(com.sun.star.frame.XFrame xFrame, 
            short sApplyTo) {
        Object oCellRange = null;
        //com.sun.star.frame.XFrame MyFrame = ListenerHelper.GetFrame();
        XController xController = xFrame.getController();
        Object oSelection = xController.getModel().getCurrentSelection();
        
        XServiceInfo xServiceInfo = (XServiceInfo)UnoRuntime.queryInterface(
            XServiceInfo.class, oSelection);
                                                                        // Calc
        if (xServiceInfo.supportsService("com.sun.star.sheet.SheetCell")  || 
            xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRange")  || 
            xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) {
            oCellRange = oSelection;
        }
                                                // selected cells in text table
        else if (xServiceInfo.supportsService("com.sun.star.text.TextTableCursor")) {
            XTextViewCursorSupplier xTextViewCursorSupplier = 
                    (XTextViewCursorSupplier)UnoRuntime.queryInterface(
                    XTextViewCursorSupplier.class, xController);
            XTextViewCursor xTextViewCursor = xTextViewCursorSupplier.getViewCursor();
            XPropertySet xPropertySet =  (XPropertySet) UnoRuntime.queryInterface
                    (XPropertySet.class, xTextViewCursor); 
                    // NB: I might be in table (has value in "TextTable" or not
            if (sApplyTo == APPLYTO_TABLE) {            // get entire table 
                try {
                     Object oUnoObject = xPropertySet.getPropertyValue("TextTable");
                     oCellRange = (XTextTable) AnyConverter.toObject
                            (XTextTable.class, oUnoObject);
                } catch (com.sun.star.lang.IllegalArgumentException ex) {
                     ex.printStackTrace();
                } catch (UnknownPropertyException ex) {
                     ex.printStackTrace();
                } catch (WrappedTargetException ex) {
                     ex.printStackTrace();
                }
            }
            else {                                    // get only selected cells
                XTextTableCursor xTextTableCursor =  (XTextTableCursor) 
                    UnoRuntime.queryInterface(XTextTableCursor.class, oSelection);
                String sSelectedRange = xTextTableCursor.getRangeName(); // cells that are selected  
                        // NB: I might be in table (has value in "TextTable" or not
                try {
                    Object oUnoObject = xPropertySet.getPropertyValue("TextTable");
                    XTextTable xTextTable = (XTextTable) AnyConverter.toObject
                        (XTextTable.class, oUnoObject);
                     if (sSelectedRange.indexOf(":") < 0) { // single cell
                        oCellRange = xTextTable.getCellByName(sSelectedRange);    
                     }
                     else { // cell range
                        XCellRange xCellRange =  (XCellRange) 
                            UnoRuntime.queryInterface(XCellRange.class, xTextTable);
                        oCellRange = xCellRange.getCellRangeByName(sSelectedRange);    
                     }
                } catch (com.sun.star.lang.IllegalArgumentException ex) {
                     ex.printStackTrace();
                } catch (UnknownPropertyException ex) {
                     ex.printStackTrace();
                } catch (WrappedTargetException ex) {
                     ex.printStackTrace();
                }
            }
        }
                                            // no selection, cursor is somewhere
        else if (xServiceInfo.supportsService("com.sun.star.text.TextRanges")) {          
            XTextViewCursorSupplier xTextViewCursorSupplier = 
                    (XTextViewCursorSupplier)UnoRuntime.queryInterface(
                    XTextViewCursorSupplier.class, xController);
            XTextViewCursor xTextViewCursor = xTextViewCursorSupplier.getViewCursor();
            XPropertySet xPropertySet =  (XPropertySet) UnoRuntime.queryInterface
                    (XPropertySet.class, xTextViewCursor); 
                    // NB: I might be in a table (has value in "TextTable" or not
            if (sApplyTo == APPLYTO_TABLE) {            // get entire table 
                try {
                     Object oUnoObject = xPropertySet.getPropertyValue("TextTable");
                     oCellRange = (XTextTable) AnyConverter.toObject
                            (XTextTable.class, oUnoObject);
                } catch (com.sun.star.lang.IllegalArgumentException ex) {
                     // this object is not a TextTable, probably somewhere in text, ignore the command
                     // ex.printStackTrace();
                } catch (UnknownPropertyException ex) {
                     ex.printStackTrace();
                } catch (WrappedTargetException ex) {
                     ex.printStackTrace();
                }
            }
            else {                                  // get cell cursor is in
                try {
                     Object oUnoObject = xPropertySet.getPropertyValue("Cell");
                     oCellRange = (XCell) AnyConverter.toObject
                        (XCell.class, oUnoObject);
                } catch (com.sun.star.lang.IllegalArgumentException ex) {
                       oCellRange = xTextViewCursor; // somewhere in text
                } catch (UnknownPropertyException ex) {
                                ex.printStackTrace();
                } catch (WrappedTargetException ex) {
                                ex.printStackTrace();
                }
            }
        }
                                                                     // in frame 
        else if (xServiceInfo.supportsService("com.sun.star.text.BaseFrame")) {
            if (sApplyTo == APPLYTO_CELL) {     // does not work for TableBorder
                oCellRange = oSelection;
            }
        }          

        return oCellRange;
    }
    
    /** Get Borders from Cells
     * @param oRange - where to get the borders from
     * @param Modifiers - always .MOD1 (ctrl)
     * @param KeyCode - NUM1 .. NUM9
     * @exception WrappedTargetException   ex.printStackTrace()
     * @exception UnknownPropertyException ex.printStackTrace()
     */
    public static void GetBordersCell(Object oRange, short Modifiers, short KeyCode) {
//        com.sun.star.table.BorderLine currentBorders  = new BorderLine();
        com.sun.star.table.BorderLine2 currentBorders2 = new BorderLine2();
        XServiceInfo xServiceInfo = (XServiceInfo)UnoRuntime.queryInterface(
            XServiceInfo.class, oRange);
        try {
            XPropertySet xTableProps = (XPropertySet)UnoRuntime.queryInterface(
                    XPropertySet.class, oRange);
            switch (KeyCode) {
                case com.sun.star.awt.Key.NUM1:     //only CALC supports diagonal lines
                    if ((xServiceInfo.supportsService("com.sun.star.sheet.SheetCell")) ||
                        (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRange")) ||
                        (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) ) 
                        {
                        //currentBorders = (BorderLine)xTableProps.getPropertyValue("DiagonalBLTR");
                        currentBorders2 = (BorderLine2)xTableProps.getPropertyValue("DiagonalBLTR");
                        }
                    break;
                case com.sun.star.awt.Key.NUM2:
                    //currentBorders = (BorderLine)xTableProps.getPropertyValue("BottomBorder");
                    currentBorders2 = (BorderLine2)xTableProps.getPropertyValue("BottomBorder");
                    break;
                case com.sun.star.awt.Key.NUM3:     //only CALC supports diagonal lines
                    if ((xServiceInfo.supportsService("com.sun.star.sheet.SheetCell")) ||
                        (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRange")) ||
                        (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) ) 
                        {
                        //currentBorders = (BorderLine)xTableProps.getPropertyValue("DiagonalTLBR");
                        currentBorders2 = (BorderLine2)xTableProps.getPropertyValue("DiagonalTLBR");
                        }
                    break;
                case com.sun.star.awt.Key.NUM4:
                    //currentBorders = (BorderLine)xTableProps.getPropertyValue("LeftBorder");
                    currentBorders2 = (BorderLine2)xTableProps.getPropertyValue("LeftBorder");
                    break;
                case com.sun.star.awt.Key.NUM6:
                    //currentBorders = (BorderLine)xTableProps.getPropertyValue("RightBorder");
                    currentBorders2 = (BorderLine2)xTableProps.getPropertyValue("RightBorder");
                    break;
                case com.sun.star.awt.Key.NUM8: 
                    //currentBorders = (BorderLine)xTableProps.getPropertyValue("TopBorder");
                    currentBorders2 = (BorderLine2)xTableProps.getPropertyValue("TopBorder");
                    break;
            }
        } catch (WrappedTargetException ex) {
            ex.printStackTrace();
        } catch (UnknownPropertyException ex) {
            ex.printStackTrace();
        }

        //jBorderLiner.TB_SelectLS.setLineNumber(GetLS(currentBorders));
//        System.out.println("GetBorderCell:="+
//        		"="+currentBorders.InnerLineWidth+"-"+currentBorders.LineDistance+"-"+
//        		currentBorders.OuterLineWidth);
//        System.out.println("GetBorderCell: "+currentBorders2.LineStyle+"-"+currentBorders2.LineWidth+
//        		"="+currentBorders2.InnerLineWidth+"-"+currentBorders2.LineDistance+"-"+
//        		currentBorders2.OuterLineWidth);
        jBorderLiner.TB_SelectLS.setLineNumber(currentBorders2.LineStyle);
        jBorderLiner.TB_SelectLW.setLineWidth(currentBorders2.LineWidth);
        jBorderLiner.TB_SelectColor.setColor(currentBorders2.Color);
    }
    
    /** Get Borders from Table
     * DEACTIVATED - requires manual conversion from Borderline2 to Borderline - not possible
     * @param oRange to get borders from
     * @param Modifiers always .MOD1 (CTRL)
     * @param KeyCode NUM1 ... NUM9
     */
    public static void GetBordersTable(Object oRange, short Modifiers, short KeyCode) {
        com.sun.star.table.BorderLine sBorderWork = new BorderLine();
        com.sun.star.table.BorderLine2 currentBorders2 = new BorderLine2();
        com.sun.star.table.TableBorder currentBorders = new TableBorder();
    	//System.out.println("GetBordersTable:");
        try {
            XPropertySet xTableProps = (XPropertySet)UnoRuntime.queryInterface(
                    XPropertySet.class, oRange);
            //System.out.println("GetBordersTable: before getPropertyValue");
            currentBorders = (TableBorder)xTableProps.getPropertyValue("TableBorder");
        } catch (WrappedTargetException ex) {
//        	System.out.println("GetBordersTable->WrappedTargetException");
            ex.printStackTrace();
        } catch (UnknownPropertyException ex) {
//        	System.out.println("GetBordersTable->UnknownPropertyException");
            ex.printStackTrace();
        }
        //System.out.println("GetBordersTable: before the switch");
        switch (KeyCode) {
            case com.sun.star.awt.Key.NUM2:
                sBorderWork = currentBorders.BottomLine;
                break;
            case com.sun.star.awt.Key.NUM4:
                sBorderWork = currentBorders.LeftLine;
                break;
            case com.sun.star.awt.Key.NUM6:
                sBorderWork = currentBorders.RightLine;
                break;
            case com.sun.star.awt.Key.NUM7:
                sBorderWork = currentBorders.HorizontalLine;
                break;
            case com.sun.star.awt.Key.NUM8:
                sBorderWork = currentBorders.TopLine;
                break;
            case com.sun.star.awt.Key.NUM9:
                sBorderWork = currentBorders.VerticalLine;
                break;
        }
//        System.out.println("GetBordersTable: before typecast");
        // currentBorders2 = (BorderLine2)sBorderWork; // causes runtime exception
        currentBorders2.Color = sBorderWork.Color;
        currentBorders2.InnerLineWidth = sBorderWork.InnerLineWidth;
        currentBorders2.LineDistance = sBorderWork.LineDistance;
        currentBorders2.OuterLineWidth = sBorderWork.OuterLineWidth;
        
        //System.out.println("GetBorderTable"+
        //		"="+sBorderWork.InnerLineWidth+"-"+sBorderWork.LineDistance+"-"+
        //		sBorderWork.OuterLineWidth);
        //System.out.println("GetBorderTable"+currentBorders2.LineStyle+"-"+currentBorders2.LineWidth+
        //		"="+currentBorders2.InnerLineWidth+"-"+currentBorders2.LineDistance+"-"+
        //		currentBorders2.OuterLineWidth);
        //	deactivated, requires manual conversion of Boderline2 to Borderline
        //jBorderLiner.TB_SelectLS.setLineNumber(currentBorders2.LineStyle);
        //jBorderLiner.TB_SelectLW.setLineWidth(currentBorders2.LineWidth);
        //jBorderLiner.TB_SelectColor.setColor(currentBorders2.Color);
        //jBorderLiner.TB_SelectLS.setLineNumber(GetLS(sBorderWork));
        //jBorderLiner.TB_SelectColor.setColor(sBorderWork.Color);
    }
    
    /** Set borders of cell or range(s) of cells
     * @param oRange
     * @param Modifiers - .SHIFT, .ALT
     * @param KeyCode - NUM1 .. NUM8
     * @exception ex.printStackTrace - NoSuchElementException, WrappedTargetException
     */
    public static void SetBordersCell(Object oRange, short Modifiers, short KeyCode) {
    	
        XServiceInfo xServiceInfo = (XServiceInfo)UnoRuntime.queryInterface(
                XServiceInfo.class, oRange);
        
        // do we have several CellRanges?, then iterate through Enumeration
        if (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) {
            XEnumerationAccess xRangeEA = (XEnumerationAccess) UnoRuntime.queryInterface(
                    XEnumerationAccess.class, oRange);
            XEnumeration xRanges = xRangeEA.createEnumeration();
            while(xRanges.hasMoreElements() ) {
                try {
                    oRange = xRanges.nextElement();
                    SetBordersCellRange(oRange, Modifiers, KeyCode);
                } catch (WrappedTargetException ex) {
                    ex.printStackTrace();
                } catch (NoSuchElementException ex) {
                    ex.printStackTrace();
                }
            }
        } 
        else SetBordersCellRange(oRange, Modifiers, KeyCode);
    }

    /** Set borders of cell or range of cells with 1 or more formats
     * @param oRange
     * @param Modifiers - .SHIFT, .ALT
     * @param KeyCode - NUM1 .. NUM8
     * @exception ex.printStackTrace - HeadlessException, WrappedTargetException,
     *      UnknownPropertyException, PropertyVetoException, 
     *      IllegalArgumentException
     */
    public static void SetBordersCellRange(Object oRange, short Modifiers, short KeyCode) {
    	
        XServiceInfo xServiceInfo = (XServiceInfo)UnoRuntime.queryInterface(
            XServiceInfo.class, oRange);
        
        //com.sun.star.table.BorderLine sBorderWork = new BorderLine(); // never read locally
        //sBorderWork = GetBorderLine(Modifiers);
        
        // if the cells have different formats, they have to be addressed individually
        if ( (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRange")) ||
             (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) ) 
             {  
                com.sun.star.sheet.XCellFormatRangesSupplier xFormatSupp =
                    (com.sun.star.sheet.XCellFormatRangesSupplier) UnoRuntime.queryInterface(
                        com.sun.star.sheet.XCellFormatRangesSupplier.class, oRange);
                com.sun.star.container.XIndexAccess xRangeIA = xFormatSupp.getCellFormatRanges();
             // use an enumAccess because IndexAccess content changes with modifying the format
             // so that getCount() value is changed. 
                XEnumerationAccess xRangeEA = (XEnumerationAccess) UnoRuntime.queryInterface(
                         XEnumerationAccess.class, xRangeIA);             
                XEnumeration xRangeE = xRangeEA.createEnumeration();
                
                while(xRangeE.hasMoreElements() )
                {
                    Object aRangeObj = null;
                    try {
                        aRangeObj = xRangeE.nextElement();
                        com.sun.star.sheet.XSheetCellRange xCellRange = (com.sun.star.sheet.XSheetCellRange)
                                UnoRuntime.queryInterface(com.sun.star.sheet.XSheetCellRange.class, aRangeObj);
                        SetBordersSingleCell(xCellRange, Modifiers, KeyCode);
                    } catch (NoSuchElementException ex) {
                        ex.printStackTrace();
                    } catch (WrappedTargetException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        else {
            SetBordersSingleCell(oRange, Modifiers, KeyCode);
        }
    }
    
    
    /** Set borders of cell or range(s) of cells
     * @param oRange
     * @param Modifiers - .SHIFT, .ALT
     * @param KeyCode - NUM1 .. NUM8
     * @exception ex.printStackTrace - HeadlessException, WrappedTargetException,
     *      UnknownPropertyException, PropertyVetoException, 
     *      IllegalArgumentException
     */
    public static void SetBordersSingleCell(Object oRange, short Modifiers, short KeyCode) {
        XServiceInfo xServiceInfo = (XServiceInfo)UnoRuntime.queryInterface(
            XServiceInfo.class, oRange);
        
        try {
            com.sun.star.table.BorderLine2 sBorderWork = new BorderLine2();
            //com.sun.star.table.BorderLine2 sBorderWork = new BorderLine2();
			sBorderWork = GetBorderLine2(Modifiers);
			//sBorderWork = (BorderLine)sBorderWork2;
                   
            XPropertySet xTableProps = (XPropertySet)UnoRuntime.queryInterface(
                XPropertySet.class, oRange);
            switch (KeyCode) {
                case com.sun.star.awt.Key.NUM1: // only Calc supports diagonal lines
                    if ( (xServiceInfo.supportsService("com.sun.star.sheet.SheetCell")) ||
                         (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRange")) ||
                         (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) ) 
                        {
                        xTableProps.setPropertyValue("DiagonalBLTR", sBorderWork);
                        }
                    break;
                case com.sun.star.awt.Key.NUM2:
                    xTableProps.setPropertyValue("BottomBorder", sBorderWork);
                    break;
                case com.sun.star.awt.Key.NUM3: // only Calc supports diagonal lines
                    if ((xServiceInfo.supportsService("com.sun.star.sheet.SheetCell")) ||
                        (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRange")) ||
                        (xServiceInfo.supportsService("com.sun.star.sheet.SheetCellRanges")) ) 
                        {
                        xTableProps.setPropertyValue("DiagonalTLBR", sBorderWork);
                        }
                    break;
                case com.sun.star.awt.Key.NUM4:
                    xTableProps.setPropertyValue("LeftBorder", sBorderWork);
                    break;
                case com.sun.star.awt.Key.NUM6:
                    xTableProps.setPropertyValue("RightBorder", sBorderWork);
                    break;
                case com.sun.star.awt.Key.NUM8:
                    xTableProps.setPropertyValue("TopBorder", sBorderWork);
                    break;
            }
            
        } catch (HeadlessException ex) {
            ex.printStackTrace();
        } catch (WrappedTargetException ex) {
            ex.printStackTrace();
        } catch (UnknownPropertyException ex) {
            ex.printStackTrace();
        } catch (com.sun.star.beans.PropertyVetoException ex) {
            ex.printStackTrace();
        } catch (com.sun.star.lang.IllegalArgumentException ex) {
            ex.printStackTrace();
        }
    }
    
    /** Set borders of table
     * @param oRange
     * @param Modifiers - .SHIFT, .ALT
     * @param KeyCode - NUM2 .. NUM9
     * @exception ex.printStackTrace - HeadlessException, WrappedTargetException,
     *      UnknownPropertyException, PropertyVetoException, 
     *      IllegalArgumentException
     */
    public static void SetBordersTable(Object oRange, short Modifiers, short KeyCode) {
        try {          
            com.sun.star.table.BorderLine sBorderWork = new BorderLine();
            com.sun.star.table.TableBorder currentBorders = new TableBorder();
            com.sun.star.table.BorderLine2 sBorderWork2 = new BorderLine2();
            
//            sBorderWork = GetBorderLine(Modifiers);
//            System.out.println("SetBorderTable:a "+
//            		"="+sBorderWork.InnerLineWidth+"-"+sBorderWork.LineDistance+"-"+
//            		sBorderWork.OuterLineWidth);
            
            sBorderWork2 = GetBorderLine2(Modifiers);
            sBorderWork = convert2toBorderLine (sBorderWork2);
            
//            System.out.println("SetBordersTable:1 "+sBorderWork2.LineStyle+"-"+sBorderWork2.LineWidth+
//            		"="+sBorderWork2.InnerLineWidth+"-"+sBorderWork2.LineDistance+"-"+
//            		sBorderWork2.OuterLineWidth);
//            System.out.println("SetBordersTable:2 "+
//            		"="+sBorderWork.InnerLineWidth+"-"+sBorderWork.LineDistance+"-"+
//            		sBorderWork.OuterLineWidth);
            
            XPropertySet xTableProps = null;
            xTableProps = (XPropertySet)UnoRuntime.queryInterface(
                XPropertySet.class, oRange);
            currentBorders = (TableBorder)xTableProps.getPropertyValue("TableBorder");
            
            switch (KeyCode) {
                case com.sun.star.awt.Key.NUM2:
                    currentBorders.BottomLine = sBorderWork;
                    currentBorders.IsBottomLineValid = true;
                    break;
                case com.sun.star.awt.Key.NUM4:
                    currentBorders.LeftLine = sBorderWork;
                    currentBorders.IsLeftLineValid = true;
                    break;
                case com.sun.star.awt.Key.NUM6:
                    currentBorders.RightLine = sBorderWork;
                    currentBorders.IsRightLineValid = true;
                    break;
                case com.sun.star.awt.Key.NUM7:
                    currentBorders.HorizontalLine = sBorderWork;
                    currentBorders.IsHorizontalLineValid = true;
                    break;
                case com.sun.star.awt.Key.NUM8:
                    currentBorders.TopLine = sBorderWork;
                    currentBorders.IsTopLineValid = true;
                    break;
                case com.sun.star.awt.Key.NUM9:
                    currentBorders.VerticalLine = sBorderWork;
                    currentBorders.IsVerticalLineValid = true;
                    break;
            }
//            System.out.println("SetBordersTable: TableBorder: "+            		currentBorders.BottomLine.InnerLineWidth + "-" + 
//            		currentBorders.BottomLine.LineDistance + "-" +
//            		currentBorders.BottomLine.OuterLineWidth +"-"+currentBorders.IsBottomLineValid);
            xTableProps.setPropertyValue("TableBorder", currentBorders);
        } catch (HeadlessException ex) {
            ex.printStackTrace();
        } catch (WrappedTargetException ex) {
            ex.printStackTrace();
        } catch (UnknownPropertyException ex) {
            ex.printStackTrace();
        } catch (com.sun.star.beans.PropertyVetoException ex) {
            ex.printStackTrace();
        } catch (com.sun.star.lang.IllegalArgumentException ex) {
            ex.printStackTrace();
        } catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    /** Get borders defined in the toolbar or zero if deleting desired
     * @param Modifiers - .SHIFT, .ALT
     */
//    public static com.sun.star.table.BorderLine GetBorderLine(short Modifiers) {
//    	com.sun.star.table.BorderLine BL = new BorderLine();
//        if (Modifiers == com.sun.star.awt.KeyModifier.SHIFT) {
//            BL.InnerLineWidth = 0; BL.OuterLineWidth = 0;
//            BL.LineDistance = 0;
//            BL.Color = 0;
//        }
//        else {
//            BL = jBorderLiner.TB_SelectLS.GetLineStyle();
//            BL.Color = jBorderLiner.TB_SelectColor.GetColor();
//        }
////        System.out.println("GetborderLine: BL: "+BL.Color+"-"+BL.InnerLineWidth+"-"+BL.LineDistance
////        		+"-"+BL.OuterLineWidth);
//        return BL;
//    }
    
    /** Get borders defined in the toolbar or zero if deleting desired (BorderLine2)
     * @param Modifiers - .SHIFT, .ALT
     */
    public static com.sun.star.table.BorderLine2 GetBorderLine2(short Modifiers) {
    	com.sun.star.table.BorderLine2 BL2 = new BorderLine2();
        if (Modifiers == com.sun.star.awt.KeyModifier.SHIFT) {
        	BL2.LineWidth = 0; BL2.LineStyle = com.sun.star.table.BorderLineStyle.NONE;
            BL2.Color = 0;
        }
        else {
            BL2.LineStyle = jBorderLiner.TB_SelectLS.GetLineStyle2();
            if (BL2.LineStyle == com.sun.star.table.BorderLineStyle.NONE) {
            	BL2.LineWidth = 0; }
            else { BL2.LineWidth = jBorderLiner.TB_SelectLW.GetLineWidth(); }
            BL2.Color = jBorderLiner.TB_SelectColor.GetColor();
        }
//        System.out.println("GetborderLine2: BL2: "+BL2.Color+"-"+BL2.LineStyle+"-"+BL2.LineWidth+
//        		BL2.InnerLineWidth+"-"+BL2.LineDistance+"-"+BL2.OuterLineWidth);
        return BL2;
    }
    
    /** find linestyle from Borderline (similar values only)
     * @param sBorderline
     * @return iLS integer
     * @deprecated not used w/BorderLine2
     */
//	private static int GetLS (com.sun.star.table.BorderLine sBorderline) {
//        int diff;
//        int diff2=9999;
//        int i=0;
//        int LSNum=0;
//        short[][] LSList = jBorderLiner.TB_SelectLS.GetLSList();
//        do {
//            diff = Math.abs(sBorderline.OuterLineWidth-LSList[i][0]) +
//                   Math.abs(sBorderline.InnerLineWidth-LSList[i][1]) +
//                   Math.abs(sBorderline.LineDistance  -LSList[i][2]);
//            if (diff < diff2) {
//                diff2 = diff;
//                LSNum = i;
//            }
//            i++;
//        }
//        while ((diff != 0) && (i<LSList.length )); 
//        return LSNum;
//    }    
	
	/** Convert BorderLine2 to BorderLine
	 * @param inBL2
	 * @return
	 */
	private static com.sun.star.table.BorderLine convert2toBorderLine 
		(com.sun.star.table.BorderLine2 inBL2) {
		com.sun.star.table.BorderLine outBL = new BorderLine();
		
		switch (inBL2.LineStyle) {
			case com.sun.star.table.BorderLineStyle.NONE:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = 0;
				outBL.LineDistance = 0;
				outBL.OuterLineWidth = 0;
				break;
			case com.sun.star.table.BorderLineStyle.SOLID:
			case com.sun.star.table.BorderLineStyle.DOTTED:
			case com.sun.star.table.BorderLineStyle.DASHED:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = 0;
				outBL.LineDistance = 0;
				outBL.OuterLineWidth = (short) inBL2.LineWidth;
				break;
			case com.sun.star.table.BorderLineStyle.DOUBLE:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = (short) inBL2.LineWidth;
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth);
				outBL.OuterLineWidth = (short) inBL2.LineWidth;
				break;
			case com.sun.star.table.BorderLineStyle.THINTHICK_SMALLGAP:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = 26;
				outBL.LineDistance = 35;
				outBL.OuterLineWidth = (short) inBL2.LineWidth;
				break;
			case com.sun.star.table.BorderLineStyle.THINTHICK_MEDIUMGAP:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = (short) (inBL2.LineWidth / 2);
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth / 2);
				outBL.OuterLineWidth = (short) (inBL2.LineWidth);
				break;
			case com.sun.star.table.BorderLineStyle.THINTHICK_LARGEGAP:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = 26;
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth);
				outBL.OuterLineWidth = 35;
				break;
			case com.sun.star.table.BorderLineStyle.THICKTHIN_SMALLGAP:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = (short) (inBL2.LineWidth);
				outBL.LineDistance = 35;
				outBL.OuterLineWidth = 26;
				break;
			case com.sun.star.table.BorderLineStyle.THICKTHIN_MEDIUMGAP:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = (short) (inBL2.LineWidth);
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth / 2);
				outBL.OuterLineWidth = (short) (inBL2.LineWidth / 2);
				break;
			case com.sun.star.table.BorderLineStyle.THICKTHIN_LARGEGAP:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = 53;
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth);
				outBL.OuterLineWidth = 26;
				break;
			case com.sun.star.table.BorderLineStyle.EMBOSSED:
			case com.sun.star.table.BorderLineStyle.ENGRAVED:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = (short) (inBL2.LineWidth / 2);
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth);
				outBL.OuterLineWidth = (short) (inBL2.LineWidth / 2);
				break;
			case com.sun.star.table.BorderLineStyle.OUTSET:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = (short) (inBL2.LineWidth);
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth);
				outBL.OuterLineWidth = 26;
				break;
			case com.sun.star.table.BorderLineStyle.INSET:
				outBL.Color = inBL2.Color;
				outBL.InnerLineWidth = 26;
				outBL.LineDistance = (short) Math.max(35, inBL2.LineWidth);
				outBL.OuterLineWidth = (short) (inBL2.LineWidth);
				break;
		}
		return outBL;
	}
	
}